import React, { createContext, useCallback, useContext, useEffect, useMemo, useState } from 'react'
import { Platform, StyleSheet, View, useWindowDimensions } from 'react-native'

import { createUUID } from '@holi/core/helpers/strings'
import HoliIcon from '@holi/ui/components/atoms/HoliIcon'
import HoliText from '@holi/ui/components/atoms/HoliText'
import SafeArea from '@holi/ui/components/atoms/SafeArea'
import HoliButton from '@holi/ui/components/molecules/HoliButton'
import { colors, vars } from '@holi/ui/styles/globalVars'

const TOAST_TIMEOUT = 2000

export enum HoliToastType {
  ERROR = 'ERROR',
  SUCCESS = 'SUCCESS',
}

interface Toast {
  id: string
  type: HoliToastType
  text: string
  timeout?: number
  testID?: string
}

export interface HoliToastProps extends Toast {
  closeToast: () => void
}

export interface ToastContextValue {
  openToast: (text: string, type?: HoliToastType, timeout?: number, testID?: string) => void
  closeToast: (id: string) => void
}

export const ToastContext = createContext<ToastContextValue>({} as ToastContextValue)

export const ToastProvider = ({ children }: React.PropsWithChildren<Partial<ToastContextValue>>) => {
  const [toasts, setToasts] = useState<Toast[]>([])
  const closeToast = useCallback(
    (id: string) => setToasts((currentToasts: Toast[]) => currentToasts.filter((toast) => toast.id !== id)),
    []
  )
  const openToast = useCallback(
    (text: string, type = HoliToastType.SUCCESS, timeout = TOAST_TIMEOUT, testID?: string) =>
      setToasts((currentToasts: Toast[]) => [...currentToasts, { id: createUUID(), text, type, timeout, testID }]),
    []
  )
  const contextValue = useMemo(() => ({ openToast, closeToast }), [closeToast, openToast])

  const { width: deviceWidth } = useWindowDimensions()
  const firstToastType = toasts?.[0]?.type
  const styles = useMemo(() => createStyles(deviceWidth, firstToastType), [deviceWidth, firstToastType])

  return (
    <ToastContext.Provider value={contextValue}>
      {children}

      {!!toasts.length && (
        <SafeArea style={[styles.wrapper, Platform.OS === 'web' && styles.wrapperWebPosition]}>
          {toasts.map((toast: Toast) => (
            <HoliToast
              key={toast.id}
              id={toast.id}
              type={toast.type}
              closeToast={() => closeToast(toast.id)}
              text={toast.text}
              timeout={toast.timeout}
              testID={toast.testID}
            />
          ))}
        </SafeArea>
      )}
    </ToastContext.Provider>
  )
}

export const useToast = () => useContext(ToastContext)

export const HoliToast: React.FC<HoliToastProps> = ({ closeToast, type, text, timeout = TOAST_TIMEOUT, testID }) => {
  const { width: deviceWidth } = useWindowDimensions()

  const styles = useMemo(() => createStyles(deviceWidth, type), [deviceWidth, type])

  useEffect(() => {
    const timer = setTimeout(closeToast, timeout)
    return () => clearTimeout(timer)
  }, [closeToast, timeout])

  return (
    <HoliButton
      transparent
      label={text}
      style={styles.toast}
      onPress={closeToast}
      testID={`toast-${testID ? testID : type}`}
    >
      <View style={styles.toastView}>
        <HoliText style={styles.toastText} selectable={false}>
          {text}
        </HoliText>
        {/* TODO: Change error icon to sad smiley */}
        <HoliIcon
          icon={type === HoliToastType.SUCCESS ? 'user-smile' : 'emotion-unhappy-noeyes'}
          size={20}
          color={colors.background10}
          style={styles.toastIcon}
        />
      </View>
    </HoliButton>
  )
}

export default HoliToast

const createStyles = (deviceWidth: number, type?: HoliToastType) =>
  StyleSheet.create({
    wrapper: {
      top: 0,
      position: 'absolute',
      zIndex: 1000,
      width: '100%',
      ...Platform.select({
        native: {
          backgroundColor: type === HoliToastType.ERROR ? colors.error30 : colors.brandColorB30,
        },
      }),
      ...(deviceWidth >= +vars.breakpointL
        ? {
            maxWidth: +vars.maxButtonWidth,
            marginLeft: 'auto',
            marginRight: 'auto',
            left: 0,
            right: 0,
          }
        : {}),
    },
    wrapperWebPosition: {
      position: 'fixed',
      width: '100%',
      top: 0,
    },
    toast: {
      padding: 20,
      flex: 1,
      backgroundColor: type === HoliToastType.ERROR ? colors.error30 : colors.brandColorB30,
      ...(deviceWidth >= +vars.breakpointL
        ? {
            borderRadius: 10,
            marginTop: 5,
          }
        : {}),
    },
    toastView: {
      display: 'flex',
      flexDirection: 'row',
      alignItems: 'center',
      justifyContent: 'center',
    },
    toastText: {
      color: colors.background10,
      textAlign: 'center',
    },
    toastIcon: {
      marginLeft: 10,
    },
  })
